From: Bart Van Assche <bvanassche@acm.org>
To: "Martin K . Petersen" <martin.petersen@oracle.com>
Cc: Jaegeuk Kim <jaegeuk@kernel.org>,
linux-scsi@vger.kernel.org, Bart Van Assche <bvanassche@acm.org>,
Hannes Reinecke <hare@suse.de>,
John Garry <john.garry@huawei.com>,
"James E.J. Bottomley" <jejb@linux.ibm.com>
Subject: [PATCH 1/2] scsi: core: Add support for reserved tags
Date: Tue, 2 Nov 2021 17:05:28 -0700 [thread overview]
Message-ID: <20211103000529.1549411-2-bvanassche@acm.org> (raw)
In-Reply-To: <20211103000529.1549411-1-bvanassche@acm.org>
Allow SCSI LLDs to allocate reserved tags by passing the BLK_MQ_REQ_RESERVED
flag to blk_get_request().
Cc: Hannes Reinecke <hare@suse.de>
Cc: John Garry <john.garry@huawei.com>
Signed-off-by: Bart Van Assche <bvanassche@acm.org>
---
drivers/scsi/scsi_lib.c | 4 +++-
include/scsi/scsi_host.h | 9 ++++++++-
2 files changed, 11 insertions(+), 2 deletions(-)
diff --git a/drivers/scsi/scsi_lib.c b/drivers/scsi/scsi_lib.c
index d0b7c6dc74f8..1cbade5fe990 100644
--- a/drivers/scsi/scsi_lib.c
+++ b/drivers/scsi/scsi_lib.c
@@ -1917,6 +1917,7 @@ int scsi_mq_setup_tags(struct Scsi_Host *shost)
{
unsigned int cmd_size, sgl_size;
struct blk_mq_tag_set *tag_set = &shost->tag_set;
+ unsigned int reserved_tags = shost->hostt->reserved_tags;
sgl_size = max_t(unsigned int, sizeof(struct scatterlist),
scsi_mq_inline_sgl_size(shost));
@@ -1932,7 +1933,8 @@ int scsi_mq_setup_tags(struct Scsi_Host *shost)
tag_set->ops = &scsi_mq_ops_no_commit;
tag_set->nr_hw_queues = shost->nr_hw_queues ? : 1;
tag_set->nr_maps = shost->nr_maps ? : 1;
- tag_set->queue_depth = shost->can_queue;
+ tag_set->queue_depth = shost->can_queue + reserved_tags;
+ tag_set->reserved_tags = reserved_tags;
tag_set->cmd_size = cmd_size;
tag_set->numa_node = NUMA_NO_NODE;
tag_set->flags = BLK_MQ_F_SHOULD_MERGE;
diff --git a/include/scsi/scsi_host.h b/include/scsi/scsi_host.h
index 97cdad14de56..5502fcb306db 100644
--- a/include/scsi/scsi_host.h
+++ b/include/scsi/scsi_host.h
@@ -367,10 +367,17 @@ struct scsi_host_template {
/*
* This determines if we will use a non-interrupt driven
* or an interrupt driven scheme. It is set to the maximum number
* of simultaneous commands a single hw queue in HBA will accept.
*/
int can_queue;
+ /*
+ * Number of tags to reserve. A reserved tag can be allocated by passing
+ * the BLK_MQ_REQ_RESERVED flag to blk_get_request().
+ */
+ unsigned reserved_tags;
+
/*
* In many instances, especially where disconnect / reconnect are
* supported, our host also has an ID on the SCSI bus. If this is
next prev parent reply other threads:[~2021-11-03 0:05 UTC|newest]
Thread overview: 22+ messages / expand[flat|nested] mbox.gz Atom feed top
2021-11-03 0:05 [PATCH 0/2] Fix a deadlock in the UFS error handler Bart Van Assche
2021-11-03 0:05 ` Bart Van Assche [this message]
2021-11-09 21:09 ` [PATCH 1/2] scsi: core: Add support for reserved tags Bart Van Assche
2021-11-10 6:59 ` Hannes Reinecke
2021-11-10 19:19 ` Bart Van Assche
2021-11-11 6:54 ` Hannes Reinecke
2021-11-15 18:46 ` Bart Van Assche
2021-11-17 0:46 ` Bart Van Assche
2021-11-03 0:05 ` [PATCH 2/2] scsi: ufs: Fix a deadlock in the error handler Bart Van Assche
2021-11-03 6:56 ` Adrian Hunter
2021-11-03 13:40 ` Bart Van Assche
2021-11-03 14:03 ` Adrian Hunter
2021-11-07 11:07 ` Avri Altman
2021-11-08 18:26 ` Bart Van Assche
2021-11-03 7:40 ` Christoph Hellwig
2021-11-03 8:37 ` Adrian Hunter
2021-11-03 16:33 ` Christoph Hellwig
2021-11-03 16:48 ` Bart Van Assche
2021-11-03 13:45 ` Bart Van Assche
2021-11-03 16:27 ` Christoph Hellwig
2021-11-03 16:39 ` Bart Van Assche
2021-11-03 16:41 ` Christoph Hellwig
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20211103000529.1549411-2-bvanassche@acm.org \
--to=bvanassche@acm.org \
--cc=hare@suse.de \
--cc=jaegeuk@kernel.org \
--cc=jejb@linux.ibm.com \
--cc=john.garry@huawei.com \
--cc=linux-scsi@vger.kernel.org \
--cc=martin.petersen@oracle.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -367,10 +367,17 @@ struct scsi_host_template {
* This determines if we will use a non-interrupt driven
* or an interrupt driven scheme. It is set to the maximum number
- * of simultaneous commands a single hw queue in HBA will accept.
+ * of simultaneous commands a single hw queue in HBA will accept. Does
+ * not include @reserved_tags.
*/
int can_queue;
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox